function [w,CPU_TIME]  = CLDA(Data, Class)
%--------------------------------------------------------------------------
% CLDA: Discriminant common vector - linear discriminant analysis
%                   (Cevikalp's fast method)
%
%   [w,CPU_TIME]  = CLDA(Data, Class)
%
%   Input:                                                             
%     Data:  data matrix (Each row is a data point) (NVec x Dim)
%     Class: class label (class 1, ..., k) 
%
%            Note: All vectors of a class is to be arranged together, e.g.,
%            if there are 2 classes with v1, v2, v3 vectors in class 1 and
%            u1, u2, u3, u4 vectors in class 2 then input Data is to be
%            formed as
%
%            Data = [v1; v2; v3; u1; u2; u3; u4],             (1)
%
%            where the size of vector v or vector u is 1xDim. Therefore the 
%            size of input Data in equation 1 would be 7xDim (depicting 
%            first 3 vectors belong to class 1 and second 4 vectors belong
%            to class 2). In the case of equation 1, class label is to be 
%            given as
%
%            Class = [1;1;1;2;2;2;2].                         (2)
%
%            size of Class (in equation 2) is 7x1.
%         
%   Output:                                                            
%          w: transformation matrix                                  
%    CPU_TIME : computational time                                 
%
%   Reference:
%   Sharma, A. and Paliwal, K.K., Linear discriminant analysis for the 
%      small sample size problem: a review, Pattern Recognition Letters
%   (in press), 2012
%
% 
%   Alok Sharma
%   University of Tokyo
%   Email: aloks@ims.u-tokyo.ac.jp
%
%--------------------------------------------------------------------------
 
t0 = cputime;
k        = max(Class); % number of classes

%-------------------------------------------------------------------------
[m,n] = size(Data);
% cc    = sum(Data)/m;

B=[];
for i = 1:k 
	loc                  = find(Class==i);
	[num,o]              = size(loc);
    vec(i)=num;
    TMP = Data(loc,1:n);
    TMP = TMP(2:end,:)-(ones(num-1,1)*TMP(1,:));
    B = [B;TMP];
end;
B=B';
[U,R] = qr(B,0);
%U=Orthonormalization(B);

for i=1:k
    mth = sum(vec(1:i-1))+1;
    x_m = Data(mth,1:n)';
    Mul = U'*x_m;
    Mul = U*Mul;
    x_com(:,i) = x_m-Mul;
end

for i=1:k-1
    b_com(:,i) = x_com(:,i+1) - x_com(:,1);
end

[w,R] = qr(b_com,0);
%w=Orthonormalization(b_com);

CPU_TIME = cputime - t0;
