function [G,TIME]  = FPILDA(Data, Class)
%--------------------------------------------------------------------------
% FPILDA Fast Pseudoinverse Linear Discriminant Analysis  (ref. Liu et al.
% 2007)
%
%   [w,CPU_TIME]  = FPILDA(Data, Class)
%
%   Input:                                                             
%     Data:  data matrix (Each row is a data point) (NVec x Dim)
%     Class: class label (class 1, ..., k) 
%
%            Note: All vectors of a class is to be arranged together, e.g.,
%            if there are 2 classes with v1, v2, v3 vectors in class 1 and
%            u1, u2, u3, u4 vectors in class 2 then input Data is to be
%            formed as
%
%            Data = [v1; v2; v3; u1; u2; u3; u4],             (1)
%
%            where the size of vector v or vector u is 1xDim. Therefore the 
%            size of input Data in equation 1 would be 7xDim (depicting 
%            first 3 vectors belong to class 1 and second 4 vectors belong
%            to class 2). In the case of equation 1, class label is to be 
%            given as
%
%            Class = [1;1;1;2;2;2;2].                         (2)
%
%            size of Class (in equation 2) is 7x1.
%         
%   Output:                                                            
%          w: transformation matrix                                  
%    CPU_TIME : computational time                                 
%
%   Reference:
%   Sharma, A. and Paliwal, K.K., Linear discriminant analysis for the 
%      small sample size problem: a review, Pattern Recognition Letters
%   (in press), 2012
%
% 
%   Alok Sharma
%   University of Tokyo
%   Email: aloks@ims.u-tokyo.ac.jp
%
%--------------------------------------------------------------------------

t0 = cputime;
class        = max(Class); % number of classes

%-------------------------------------------------------------------------
[m,n] = size(Data);
cc    = sum(Data)/n;

for i = 1:class
	loc   = find(Class==i);
	[num,col] = size(loc);
    nvec(i)=num;
end
mu = mean(Data);
Hb=[]; Hw=[];
for i=1:class
    rg = 1+sum(nvec(1:i-1)):sum(nvec(1:i));
    x = Data(rg,:);
    mui = mean(x);
    Hb = [Hb, sqrt(nvec(i))*(mui-mu)'];
    Hw = [Hw, (x - ones(nvec(i),1)*mui)'];
end


%Step 1
[Q1,D1]=svd(Hw,0);
rw = rank(D1);
D1 = D1(1:rw,1:rw);
Q1 = Q1(:,1:rw);

%Step 2
Hbh = Q1'*Hb;

%Step 3
R = Hbh*Hbh';
R = (D1^-2)*R;
[U,Del]=eig(R);
rb=rank(Del);
d=diag(Del);
[row,col]=sort(d);
len=length(row);
U=U(:,col(len-rb+1:len));

%Step 4
W = Q1*U(:,1:rb);

G = W;
TIME = cputime - t0;
