function [G,CPU_TIME]  = QRNLDA(Data, Class)
%--------------------------------------------------------------------------
% QRNLDA: QR - Null linear discriminant analysis
%
%   [w,CPU_TIME]  = QRNLDA(Data, Class)
%
%   Input:                                                             
%     Data:  data matrix (Each row is a data point) (NVec x Dim)
%     Class: class label (class 1, ..., k) 
%
%            Note: All vectors of a class is to be arranged together, e.g.,
%            if there are 2 classes with v1, v2, v3 vectors in class 1 and
%            u1, u2, u3, u4 vectors in class 2 then input Data is to be
%            formed as
%
%            Data = [v1; v2; v3; u1; u2; u3; u4],             (1)
%
%            where the size of vector v or vector u is 1xDim. Therefore the 
%            size of input Data in equation 1 would be 7xDim (depicting 
%            first 3 vectors belong to class 1 and second 4 vectors belong
%            to class 2). In the case of equation 1, class label is to be 
%            given as
%
%            Class = [1;1;1;2;2;2;2].                         (2)
%
%            size of Class (in equation 2) is 7x1.
%         
%   Output:                                                            
%          w: transformation matrix                                  
%    CPU_TIME : computational time                                 
%
%   Reference:
%   Sharma, A. and Paliwal, K.K., Linear discriminant analysis for the 
%      small sample size problem: a review, Pattern Recognition Letters
%   (in press), 2012
%
% 
%   Alok Sharma
%   University of Tokyo
%   Email: aloks@ims.u-tokyo.ac.jp
%
%--------------------------------------------------------------------------
 
t0 = cputime;
k        = max(Class); % number of classes

%-------------------------------------------------------------------------
[n,dim] = size(Data);

for i = 1:k 
	loc      = find(Class==i);
	[num,col]  = size(loc);
    
    Num(i)=num;
    wi= ones(num,1);
    [wi,ri]=qr(wi);
    wi=-wi;
    rg=sum(Num(1:i-1));
    Wi(1+rg:num+rg,1+rg:num+rg)=wi;
    w(i,1)=sqrt(num);
end;
[w,r]=qr(w);
w=-w;
W=eye(n);
W(1:k,1:k)=w;

P=eye(n);
for j=1:k
    s=sum(Num(1:j-1))+1;
    tmp=P(:,j);
    P(:,j)=P(:,s);
    P(:,s)=tmp;
end


[U1,R]=qr(Data',0);

Z=R*Wi*P*W;
R2=Z(:,2:k);
R3=Z(:,k+1:n);
R32=[R3,R2];
[Q,R]=qr(R32,0);
q=rank(R3);
y=rank(R32);
Q2=Q(:,y-(y-q)+1:y);
G=U1*Q2;

CPU_TIME = cputime - t0;
