function [w,CPU_TIME] = Range_Sb(Data, Class)
%--------------------------------------------------------------------------
% Range_Sb Range Space of between-class scatter matrix 
%
%   [w,CPU_TIME]  = Range_Sb(Data, Class)
%
%   Input:                                                             
%     Data:  data matrix (Each row is a data point) (NVec x Dim)
%     Class: class label (class 1, ..., k) 
%
%            Note: All vectors of a class is to be arranged together, e.g.,
%            if there are 2 classes with v1, v2, v3 vectors in class 1 and
%            u1, u2, u3, u4 vectors in class 2 then input Data is to be
%            formed as
%
%            Data = [v1; v2; v3; u1; u2; u3; u4],             (1)
%
%            where the size of vector v or vector u is 1xDim. Therefore the 
%            size of input Data in equation 1 would be 7xDim (depicting 
%            first 3 vectors belong to class 1 and second 4 vectors belong
%            to class 2). In the case of equation 1, class label is to be 
%            given as
%
%            Class = [1;1;1;2;2;2;2].                         (2)
%
%            size of Class (in equation 2) is 7x1.
%         
%   Output:                                                            
%          w: transformation matrix                                  
%    CPU_TIME : computational time                                 
%
%   Reference:
%   Sharma, A. and Paliwal, K.K., Linear discriminant analysis for the 
%      small sample size problem: a review, Pattern Recognition Letters
%   (in press), 2012
%
% 
%   Alok Sharma
%   University of Tokyo
%   Email: aloks@ims.u-tokyo.ac.jp
%
%--------------------------------------------------------------------------

t0 = cputime;

%------------------------------ START -------------------------------%
c        = max(Class); % number of classes
[Nvec,dim] = size(Data);
mu    = sum(Data)/Nvec;

for i = 1:c
	loc     = find(Class==i);
	[nvec(i),cl] = size(loc);
end

% STEP 1: Pre-processing stage
Ht = (Data - ones(Nvec,1)*mu)';
[Ut,Dt] = svd(Ht,0);
rt=rank(Dt);
Ut = Ut(:,1:rt);
m = Ut'*Data';
m=m';
mu = mean(m);
Hb=[];
for i=1:c
    rg = 1+sum(nvec(1:i-1)):sum(nvec(1:i));
    x = m(rg,:);
    mui = mean(x);
    Hb = [Hb, sqrt(nvec(i))*(mui-mu)'];
end
rb = rank(Hb);

[Ub,Db] = svd(Hb,0);
w = Ub(:,1:rb);

% Step 2
w = Ut*w;
%------------------------------  END  -------------------------------%

CPU_TIME = cputime - t0;
end

